***********************************************************************
*                                                                     *
* Program......: AKMENU.PRG                                           *
* System.......: XBase++ Investigation                                *
* Written By...: Ashok Kumar, ISL NZ Limited, DUNEDIN, NEW ZEALAND    *
*                ashok.kumar@islnz.co.nz                              *
* Date Written.: March, 1998                                          *
* Revisions....:                                                      *
                                                                      *
* Description..: PROMPT/MENU/ALERT/CRT WRAPPER                        *
*                This file contails functions to allow migration of   *
*                Clipper ALERT and PROMPT/MENU TO to XBase CRT mode   *
*                                                                     *
* Functions....: CRTBOX() - Modal CRT Box to be used as wrapper for   *
*                           existing functions                        *
*                         - Unmodified CRTBOX.PRG from Alaska         *
*                                                                     *
*                ButtonMenuTo() - Replacement for PROMPT/MENU TO      *
*                           Add  #include "MENUTO.CH" BEFORE          *
*                           all your prompts                          *
*                         - Modified MENUTO.PRG from Alaska to positio*
*                           the buttons for each prompt allowing      *
*                           horizontal and vertical                   *
*                                                                     *
*                KeyHandler() - Key handler for ButtonMenu().         *
*                         - Modified MENUTO.PRG from Alaska to allow  *
*                           SETKEY() code blocks                      *
*                                                                     *
*                Alert()  - Replacement Clipper Alert function to     *
*                           use windows wrapper and pushbuttons       *
*                           Compile this and all your alerts will be  *
*                           changed. This is a wrapper function       *
*                                                                     *
*                ak_alert() - Actual alert function called from the   *
*                           wrapper function                          *
*                                                                     *
* Examples.....: ALERT("Welcome to XBase++")                          *
*                if ALERT("Confirm ?", { "Yes", "No"} ) = 1           *
*                                                                     *
*                #include "MENUTO.CH"                                 *
*                @10,10 PROMPT "Menu Item 1"                          *
*                @10,10 PROMPT "Menu Item 2"                          *
*                @10,10 PROMPT "Menu Item 2"                          *
*                MenuChoice := 1                                      *
*                MENU TO MenuChoice                                   *
*                                                                     *
***********************************************************************


//////////////////////////////////////////////////////////////////////
//
//  CRTBOX.PRG
//
//  Copyright:
//     Alaska Software Inc., (c) 1997. All rights reserved.         
//  
//  Contents:
//     The example demonstrates with Achoice() and MemoEdit() how
//     single procedures of a program can be encapsulated in a nodal
//     XbpCrt window when an existing application is migrated to GUI.
//  
//  
//////////////////////////////////////////////////////////////////////

#include "Common.ch"
#include "Inkey.ch"
#include "Gra.ch"
#include "Xbp.ch"

/*
 * All procedures of a program using a popup window (i.e. embedded within
 * Savescreen() / Restscreen()) can be run in a modal XbpCrt window.
 */
FUNCTION CrtBox( nT, nL, nB, nR, cTitle, bAction )
   LOCAL oAppWindow, cColor, nCursor, oError
   LOCAL oCrt, nRowCount, nColCount, nX, nY, xReturn

   oAppWindow := SetAppWindow()

   IF .NOT. oAppWindow:isDerivedFrom( "XbpCrt" )
     /*
      * This function requires SetAppWindow() to return a XbpCrt window
      */
      oError               := Error():new()
      oError:description   := "SetAppwindow() must return an XbpCrt window"
      oError:canDefault    := .F.
      oError:canRetry      := .F.
      oError:canSubstitute := .F.
      oError:operation     := "CrtBox"
      oError:args          := { nT, nL, nB, nR, cTitle, bAction }
      Break( oError )
   ENDIF
   
   DEFAULT nT      TO 0        , ;
           nL      TO 0        , ;
           nB      TO MaxRow() , ;
           nR      TO MaxCol() , ;
           cTitle  TO " "

  /*
   * Calculate graphical coordinates from text coordinates (row/col -> x/y)
   */
   nRowCount  := nB - nT + 1
   nColCount  := nR - nL + 1
   nX         := nL * oAppWindow:fontWidth
   nY         := ( MaxRow() - nB ) * oAppWindow:fontHeight
   nX         += oAppWindow:xPos
   nY         += oAppWindow:yPos
   cColor     := SetColor()
   nCursor    := SetCursor()

  /*
   * Create new XbpCrt window
   */
   oCrt := XbpCrt():new( AppDeskTop(), oAppWindow, {nX,nY}, nRowCount, nColCount, cTitle )
   oCrt:minmax       := .F.
   oCrt:sysmenu      := .F.
   oCrt:closeable    := .F.
   oCrt:clipChildren := .F.
   oCrt:border       := XBPDLG_RECESSEDBORDERTHICK_FIXED
   oCrt:fontName     := oAppwindow:fontName
   oCrt:fontHeight   := oAppWindow:fontHeight
   oCrt:fontWidth    := oAppWindow:fontWidth 
   oCrt:create()

   SetAppWindow( oCrt )
   SetColor( "N/W" )
   CLS

   SetColor ( cColor  )
   SetCursor( nCursor )
   SetMouse ( .T. )

  /*
   * Switch to modal state and give focus
   */
   oCrt:setModalState( XBP_DISP_APPMODAL )
   SetAppFocus( oCrt )

  /*
   * Procedure is called from the received code block
   */
   xReturn := Eval( bAction )

  /*
   * Clean up
   */
   oCrt:setModalState( XBP_DISP_MODELESS )
   oCrt:destroy()

   SetAppWindow( oAppWindow )
   SetAppFocus ( oAppWindow )
   SetMouse ( .F. )

RETURN xReturn
//////////////////////////////////////////////////////////////////////

/*
 * The function creates one pushbutton for each @ PROMPT command
 * and assumes the following to position the buttons:
 *
 *   - text coordinates are ignored. Instead, pushbuttons are 
 *     displayed centered in the window from top to bottom
 *
 *   - a pushbutton uses two text rows as height
 * 
 *   - the width of a pushbutton is calculated from font width
 * 
 *   - a vertical distance between buttons is calculated from font height
 *     and number of buttons
 */
FUNCTION ButtonMenuTo( aPromptList, nVar, cVarName )
   LOCAL nEvent, mp1, mp2, oXbp, aPos, aSize, cPrompt
   LOCAL oFrame, oMsg
   LOCAL nCount      := Len( aPromptList )
   LOCAL nFontHeight := SetAppWindow():fontHeight
   LOCAL nFontWidth  := SetAppWindow():fontWidth
   LOCAL aButtons[ nCount ]
   LOCAL nDistance, nHeight, nWidth
   LOCAL i, lExit, bActivate, bKeyHandler

   DEFAULT nVar TO 1  
   ReadVar( cVarName )
   
  /*
   * Height of one pushbutton
   */
   nHeight := 1.5 * nFontheight

  /*
   * Width of one pushbutton
   */
   nWidth := ( Len( aPromptList[1,3] ) ) * nFontwidth

  /*
   * Vertical distance between pushbuttons
   */
   nDistance := 3

  /*
   * Size of pushbuttons
   */
   aSize := { nWidth, nHeight }

  /*
   * Calculate centered position for first pushbutton
   */
   aPos    := Array( 2 )
   *** aPos[1] := ( aPromptList[1,2] + 1 ) * nFontWidth
   *** aPos[2] := ( MaxRow()-aPromptList[1,1] + 1 ) * (nFontHeight)

  /*
   * Callback code blocks for pushbuttons
   */
   bKeyHandler := {|mp1,mp2,obj| KeyHandler( mp1, obj, aButtons, aPromptList, oMsg, @lExit ) }
   bActivate   := {|mp1,mp2,obj| lExit := .T., nVar := AScan( aButtons, obj ) }

  /*
   * Create the pushbuttons. :cargo stores the short-cut letter
   */
   FOR i:=1 TO nCount
      cPrompt              := LTrim( aPromptList[i,3] )
      IF .NOT. "~" $ cPrompt
         cPrompt := "~" + cPrompt
      ENDIF

      aPos[1] := ( aPromptList[i,2] + 1 ) * nFontWidth
      aPos[2] := ( MaxRow()-aPromptList[i,1] -1 ) * (nHeight + nDistance) + 10

      aSize := { Len( aPromptList[i,3] ) * nFontwidth, nHeight }

      aButtons[i]          := XbpPushbutton():new( ,, aPos, aSize )
      aButtons[i]:caption  := cPrompt
      aButtons[i]:cargo    := Upper( SubStr( cPrompt, At( "~", cPrompt)+1, 1 ) )
      aButtons[i]:create()

     /*
      * Assign callback code blocks for event handling
      */
      aButtons[i]:activate := bActivate
      aButtons[i]:keyboard := bKeyHandler

   NEXT

   IF Set( _SET_MESSAGE ) > 0
     /*
      * Frame for statusbar
      */
      oFrame      := XbpStatic():new(,, {0,0}, { (MaxCol()+1)*nFontWidth, nFontHeight*1.5} )
      oFrame:type := XBPSTATIC_TYPE_RAISEDBOX
      oFrame:create()

     /*
      * Status bar is child of frame
      */
      aSize    := oFrame:currentSize()
      aSize[1] -= 2
      aSize[2] -= 2

      oMsg         := XbpStatic():new( oFrame,, {1,1}, aSize )
      oMsg:type    := XBPSTATIC_TYPE_TEXT
      oMsg:options := XBPSTATIC_TEXT_VCENTER + XBPSTATIC_TEXT_CENTER
      oMsg:caption := aPromptList[ nVar , 4 ]
      oMsg:create()
      oMsg:setColorBG( GRA_CLR_PALEGRAY )
      oMsg:setColorFG( GRA_CLR_RED )
   ENDIF

  /*
   * Set focus to initial "prompt" pushbutton
   */
   SetAppFocus( aButtons[nVar] )
   lExit := .F.
   nVar  := 0

   DO WHILE ! lExit 
      nEvent := AppEvent( @mp1, @mp2, @oXbp )
      oXbp:handleEvent( nEvent, mp1, mp2 )
   ENDDO

  /*
   * release system resources
   */
   AEval( aButtons, {|o| o:destroy() } )

   IF oFrame <> NIL
      oFrame:destroy()
     /*
      * oMsg is a child of oFrame and is destroyed implicitly
      */
   ENDIF

   ReadVar( "" )
   ASize( aPromptList, 0 )

RETURN nVar



/*
 * Process keys not understood by XbpPushbutton
 */
STATIC PROCEDURE KeyHandler( nKey, obj, aButtons, aPromptList, oMsg, lExit )
   LOCAL nCurrent := AScan( aButtons, obj ), cChar := Upper( Chr(nKey) )

   IF ( bkeyblock := SETKEY( nKey ) ) != nil
      EVAL( bkeyblock, PROCNAME( 2 ), PROCLINE( 2 ) )
      RETURN
   ENDIF

   DO CASE
   CASE nKey == xbeK_UP .OR. nKey == xbeK_LEFT
      nCurrent --

   CASE nKey == xbeK_DOWN .OR. nKey == xbeK_RIGHT
      nCurrent ++

   CASE nKey == xbeK_RETURN .OR. nKey == xbeK_PGDN
      PostAppEvent( xbeP_Activate,,, obj )
      nCurrent := -1

   CASE nKey == xbeK_ESC
     /*
      * lExit is passed by reference from the bKeyHandler codeblock
      * This terminates the event loop
      */
      lExit    := .T.
      nCurrent := -1

   CASE ( cChar >= "A" .AND. cChar <= "Z" ) .OR. cChar $ "0123456789"
     /*
      * Selection via short-cut key
      */
      nCurrent := AScan( aButtons, {|o| o:cargo == cChar } )

      IF nCurrent > 0
         PostAppEvent( xbeP_Activate,,, aButtons[ nCurrent ] )
      ELSE
         nCurrent := -1
      ENDIF

   ENDCASE

   IF nCurrent <> -1
     /*
      * Find next next pushbutton
      */
      IF nCurrent < 1
         IF Set( _SET_WRAP )
            nCurrent := Len( aButtons )
         ELSE
            nCurrent := 1
         ENDIF

      ELSEIF nCurrent > Len( aButtons )
         IF Set( _SET_WRAP )
            nCurrent := 1
         ELSE
            nCurrent := Len( aButtons )
         ENDIF
      ENDIF

     /*
      * Set focus to next pushbutton
      */
      SetAppFocus( aButtons[ nCurrent ] )

     /*
      * Display MESSAGE text
      */
      IF oMsg <> NIL
         oMsg:setCaption( aPromptList[ nCurrent, 4 ] )
      ENDIF
   ENDIF

RETURN
//////////////////////////////////////////////////////////////////////



/*
 * Wrapper function for a popup window that runs alert()
 */
FUNCTION alert(amsg,aopts)

   LOCAL bAction, xReturn
   LOCAL aopta, Achoice, nax, a2, axstart, axstarto, alen, nAT, nal, nab, nar
   LOCAL amsg2 := '', amsg3 := '', savcol := SETCOLOR(), aoptx := '', amsg1 := amsg

   amsg = Alltrim(amsg)
   a2   = AT(";", amsg)
   IF a2 > 0
      amsg2 = SUBSTR(amsg, a2+1)
      amsg1  = LEFT(amsg, a2-1)
   ENDIF
   a2   = AT(";", amsg2)
   IF a2 > 0
      amsg3 = SUBSTR(amsg2, a2+1)
      amsg2  = LEFT(amsg2, a2-1)
   ENDIF

   IF aopts == nil .OR. EMPTY(aopts)
      aopta = {'Ok'}
   ELSE
      aopta = aopts
   ENDIF

   alen = (LEN(aopta)-1) * 3
   FOR nax = 1 TO LEN(aopta)
      aoptx = aoptx + LEFT(aopta[nax],1)
      aopta[nax] = '  ' + Alltrim(aopta[nax]) + '  '
      alen = alen + LEN(aopta[nax])
   NEXT
   axstart = 39 - INT(alen/2)
   aoptx = UPPER(aoptx)

   nax = MAX(LEN(amsg1), LEN(amsg2)) ; nax = MAX(nax, alen) ; nax = INT((nax+8)/2)
   nAT = 10 ; nal = 39 - nax ; nab = 14 ; nar = 39 + nax
   IF .NOT.EMPTY(amsg2)
      nAT = nAT - 1
   ENDIF
   IF .NOT.EMPTY(amsg3)
      nAT = nAT - 1
   ENDIF

   bAction := {|| ak_alert(amsg,aopts) }

   cTitle  := "Alert"

   xReturn := CrtBox( nat, nal, nab, nar, cTitle, bAction )

RETURN xReturn


*!*****************************************************************************
*!
*!       Function: ak_alert()
*!
*!*****************************************************************************
FUNCTION ak_alert(amsg,aopts)

   LOCAL aopta, Achoice, nax, a2, axstart, axstarto, alen, nAT, nal, nab, nar, savscr
   LOCAL amsg2 := '', amsg3 := '', savcol := SETCOLOR(), aoptx := '', amsg1 := amsg

   amsg = Alltrim(amsg)
   a2   = AT(";", amsg)
   IF a2 > 0
      amsg2 = SUBSTR(amsg, a2+1)
      amsg1  = LEFT(amsg, a2-1)
   ENDIF
   a2   = AT(";", amsg2)
   IF a2 > 0
      amsg3 = SUBSTR(amsg2, a2+1)
      amsg2  = LEFT(amsg2, a2-1)
   ENDIF

   IF aopts == nil .OR. EMPTY(aopts)
      aopta = {'Ok'}
   ELSE
      aopta = aopts
   ENDIF

   alen = (LEN(aopta)-1) * 3
   FOR nax = 1 TO LEN(aopta)
      aoptx = aoptx + LEFT(aopta[nax],1)
      aopta[nax] = '  ' + Alltrim(aopta[nax]) + '  '
      alen = alen + LEN(aopta[nax])
   NEXT
   axstart = 39 - INT(alen/2)
   aoptx = UPPER(aoptx)

   nax = MAX(LEN(amsg1), LEN(amsg2)) ; nax = MAX(nax, alen) ; nax = INT((nax+8)/2)
   nAT = 10 ; nal = 39 - nax ; nab = 14 ; nar = 39 + nax
   IF .NOT.EMPTY(amsg2)
      nAT = nAT - 1
   ENDIF
   IF .NOT.EMPTY(amsg3)
      nAT = nAT - 1
   ENDIF

   SET COLOR TO GR+/R
   CLEAR
   nAb := nAb - nAT
   nAT := 0
   nAR := nAR - nal
   nal := 0

   axstart = (nar/2) - INT(alen/2)

   @nAT+1, (nar/2)-INT(LEN(amsg)/2) SAY amsg
   IF .NOT.EMPTY(amsg2)
      @nAT+2, (nar/2)-INT(LEN(amsg2)/2) SAY amsg2
   ENDIF
   IF .NOT.EMPTY(amsg3)
      @nAT+3, (nar/2)-INT(LEN(amsg3)/2) SAY amsg3
   ENDIF

   #include "Menuto.ch"
   axstarto = axstart
   DO WHILE .T.
      axstart = axstarto
      FOR nax = 1 TO LEN(aopta)
         @ nab-1, axstart PROMPT aopta[nax]
         axstart = axstart + LEN(aopta[nax]) + 3
      NEXT
      Achoicex := 1
      MENU TO Achoicex

      //IF LASTKEY() = 13 .OR. UPPER(CHR(LASTKEY())) $ aoptx .OR. LASTKEY() = 27
         EXIT
      // ENDIF
   ENDDO
   SET COLOR TO (savcol)
RETURN Achoicex



